/* $Id: sinfo_rec_lingain.c,v 1.21 2009/09/11 10:14:37 amodigli Exp $
 *
 * This file is part of the DETMON Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: amodigli $
 * $Date: 2009/09/11 10:14:37 $
 * $Revision: 1.21 $
 * $Name: sinfo-2_2_5 $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*----------------------------------------------------------------------------
                             Includes and Defines
 ----------------------------------------------------------------------------*/

#include <cpl.h>
#include "sinfo_irplib_cpl_wrp.h"

#include "sinfo_msg.h"
#include "sinfo_utils_wrappers.h"
#include "sinfo_error.h"

/* Define here the DO.CATG keywords */
#define SINFO_LINGAIN_ON_RAW                    "LINEARITY_LAMP_ON"
#define SINFO_LINGAIN_OFF_RAW			"LINEARITY_LAMP_OFF"

#define RECIPE_NAME "sinfo_rec_lingain"

/* Copy here instrument specific keywords which need to be in the PAF file */
#define INSTREGEXP   "ESO INS SETUP ID"

#define PAFREGEXP    "^(" REGEXP "|" INSTREGEXP ")$"
#define NIR TRUE
#define SINFO_BPMBIN 1
#define SINFO_KAPPA 9


/*----------------------------------------------------------------------------
			     Functions prototypes
 ----------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/**
 * @defgroup sinfo_rec_lingain Recipe to monitor detector's linearity and gain
 */
/*---------------------------------------------------------------------------*/
/**@{*/

cpl_error_code
sinfo_lingain_fill_parlist_default(cpl_parameterlist * parlist);

CPL_RECIPE_DEFINE(sinfo_rec_lingain, SINFONI_BINARY_VERSION,
		     /* Replace DETMON_BINARY_VERSION with
			corresponding pipeline macro */
                     sinfo_lingain_fill_parlist_default(recipe->parameters),
		     /* Replace "sinfoni" with PACKAGE_TARNAME
			when moved into SINFONI */
                     "Lander de Bilbao", "lbilbao@eso.org", "2008",
                     "Linearity/Gain recipe for the IR domain",
                     LG_DESCR(RECIPE_NAME, "SINFONI",
                                                      SINFO_LINGAIN_ON_RAW,
                                                      SINFO_LINGAIN_OFF_RAW));

/*---------------------------------------------------------------------------*/
/*
  @brief    Interpret the command line options and execute the data processing
  @param    frameset    the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/

static int sinfo_rec_lingain(cpl_frameset            * frameset,
			 const cpl_parameterlist * parlist)
{
  const char* name_o=NULL;
  cpl_frame* frm=NULL;
  cpl_image* ima=NULL;
  cpl_error_code error = CPL_ERROR_NONE;
  cpl_propertylist* plist=NULL;
   cpl_propertylist * lintbl;
   cpl_propertylist * gaintbl;
   cpl_propertylist * coeffscube;
   cpl_propertylist * bpm;
   cpl_propertylist * corr;
   cpl_propertylist * diff_flat;


   lintbl =  DETMON_FILL_PROLIST("DET_LIN_INFO", "TYPE", "TECH", CPL_TRUE);
   gaintbl = DETMON_FILL_PROLIST("GAIN_INFO",    "TYPE", "TECH", CPL_TRUE);
   coeffscube = DETMON_FILL_PROLIST("COEFFS_CUBE",  "TYPE", "TECH", CPL_TRUE);
   bpm = DETMON_FILL_PROLIST("BP_MAP_NL",    "TYPE", "TECH", CPL_TRUE);
   corr = DETMON_FILL_PROLIST("AUTOCORR",     "TYPE", "TECH", CPL_TRUE);
   diff_flat = DETMON_FILL_PROLIST("DIFF_FLAT",    "TYPE", "TECH", CPL_TRUE);

   error = DETMON_LG(frameset,
                                                  parlist,
                                                  SINFO_LINGAIN_ON_RAW,
                                                  SINFO_LINGAIN_OFF_RAW,
                                                  RECIPE_NAME,
    /* Replace this string ("sinfoni") with */    PACKAGE_TARNAME,
    /* PACKAGE_TARNAME when moved into SINFONI */ PAFREGEXP,
                                                  lintbl, gaintbl,
	                                          coeffscube, bpm,
						  corr, diff_flat,
                                                  PACKAGE "/" PACKAGE_VERSION,
                                                  NULL, NULL, NIR);
    sinfo_free_propertylist(&lintbl);
    sinfo_free_propertylist(&gaintbl);
    sinfo_free_propertylist(&coeffscube);
    sinfo_free_propertylist(&bpm);
    sinfo_free_propertylist(&corr);
    sinfo_free_propertylist(&diff_flat);

    check_nomsg(frm=cpl_frameset_find(frameset,"BP_MAP_NL"));
    check_nomsg(name_o=cpl_frame_get_filename(frm));
    check_nomsg(plist=cpl_propertylist_load(name_o,0));
    check_nomsg(ima=cpl_image_load(name_o,CPL_TYPE_FLOAT,0,0));
    check_nomsg(cpl_image_threshold(ima,0.1,0.9,1,0));
    check_nomsg(cpl_image_save(ima,name_o,CPL_BPP_IEEE_FLOAT,plist,CPL_IO_DEFAULT));

  cleanup:
    sinfo_free_propertylist(&plist);
    sinfo_free_image(&ima);

    /* Propagate the error, if any */
    cpl_ensure_code(!error, error);

    return CPL_ERROR_NONE;
}

cpl_error_code
sinfo_lingain_fill_parlist_default(cpl_parameterlist * parlist)
{

   cpl_error_code error = CPL_ERROR_NONE;
   cpl_parameter * p;
   error = DETMON_LG_FILL_PARLIST_NIR_DEFAULT(parlist,RECIPE_NAME,
                                                 PACKAGE_TARNAME);
    cpl_ensure_code(!error, error);
    p = cpl_parameterlist_find(parlist, PACKAGE_TARNAME "." RECIPE_NAME ".bpmbin");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_bool(p, SINFO_BPMBIN);
    p = cpl_parameterlist_find(parlist, PACKAGE_TARNAME "." RECIPE_NAME ".kappa");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_double(p, SINFO_KAPPA);
    cpl_ensure_code(!error, error);
    return error;

}

